// ScopeView.cpp : implementation of the CScopeView class
//

#include "stdafx.h"
#include "Scope.h"
#include "math.h"

#include "ScopeDoc.h"
#include "ScopeView.h"
#include "GridConfigDlg.h"
#include "ChildFrm.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CScopeView

IMPLEMENT_DYNCREATE(CScopeView, CView)

BEGIN_MESSAGE_MAP(CScopeView, CView)
	//{{AFX_MSG_MAP(CScopeView)
	ON_WM_TIMER()
	ON_WM_DESTROY()
	ON_WM_SIZE()
	ON_WM_PAINT()
	ON_WM_LBUTTONDOWN()
	ON_WM_LBUTTONUP()
	ON_WM_MOUSEMOVE()
	ON_COMMAND(ID_EDIT_COPY, OnEditCopy)
	ON_WM_CLOSE()
	//}}AFX_MSG_MAP
	// Standard printing commands
	ON_COMMAND(ID_FILE_PRINT, CView::OnFilePrint)
	ON_COMMAND(ID_FILE_PRINT_DIRECT, CView::OnFilePrint)
	ON_COMMAND(ID_FILE_PRINT_PREVIEW, CView::OnFilePrintPreview)
END_MESSAGE_MAP()

















/////////////////////////////////////////////////////////////////////////////
// CScopeView construction/destruction

CScopeView::CScopeView()
{
	m_bitmapCH1 = NULL;

	m_nClientHeight	= 0;
	m_nClientWidth = 0;
	x = 100;
	y = 80;
		
	m_nCH1POS = 40;
	m_nCH2POS = 20;
	m_nCH3POS = 50;
	m_nCH4POS = 60;

	m_nCH1_Offset = 0;
	m_nCH2_Offset = 0;
	m_nCH3_Offset = 0;
	m_nCH4_Offset = 0;

	m_nTriggerCH1 = 0;
	m_nVPD_CH1 = 100; /* 1 voltage per division */
	m_nVPD_CH2 = 100;
	m_nVPD_CH3 = 100;
	m_nVPD_CH4 = 100;

	penGrid =NULL;
	penCH1 = NULL;
	penCH2 = NULL;
	penCH3 = NULL;
	penCH4 = NULL;

	m_bChangeZeroCH1 = false;
	m_bChangeZeroCH2 = false;
	m_bChangeZeroCH3 = false;
	m_bChangeZeroCH4 = false;

	m_bChangeOffsetCH1 = false;

	m_nRefreshRate = 200; // Default refresh rate 200ms
}

CScopeView::~CScopeView()
{


}

BOOL CScopeView::PreCreateWindow(CREATESTRUCT& cs)
{
	// TODO: Modify the Window class or styles here by modifying
	//  the CREATESTRUCT cs

	return CView::PreCreateWindow(cs);
}

/////////////////////////////////////////////////////////////////////////////
// CScopeView drawing


void CScopeView::OnDraw(CDC* pDC)
{
	CScopeDoc* pDoc = GetDocument();
	ASSERT_VALID(pDoc);


}

/////////////////////////////////////////////////////////////////////////////
// CScopeView printing

BOOL CScopeView::OnPreparePrinting(CPrintInfo* pInfo)
{			
	pInfo->m_nCurPage = 1;	// Current page set as page 1
	pInfo->SetMaxPage(1);	// Number of pages to print is always 2
	return DoPreparePrinting(pInfo);
}

void CScopeView::OnBeginPrinting(CDC* /*pDC*/, CPrintInfo* /*pInfo*/)
{
	// TODO: add extra initialization before printing
}

void CScopeView::OnEndPrinting(CDC* /*pDC*/, CPrintInfo* /*pInfo*/)
{
	// TODO: add cleanup after printing
}

/////////////////////////////////////////////////////////////////////////////
// CScopeView diagnostics

#ifdef _DEBUG
void CScopeView::AssertValid() const
{
	CView::AssertValid();
}

void CScopeView::Dump(CDumpContext& dc) const
{
	CView::Dump(dc);
}

CScopeDoc* CScopeView::GetDocument() // non-debug version is inline
{
	ASSERT(m_pDocument->IsKindOf(RUNTIME_CLASS(CScopeDoc)));
	return (CScopeDoc*)m_pDocument;
}
#endif //_DEBUG

/////////////////////////////////////////////////////////////////////////////
// CScopeView message handlers


void CScopeView::OnTimer(UINT nIDEvent) 
{
	if (nIDEvent == nTimerRefresh)
	{
		KillTimer(nTimerRefresh);

		Trigger();	/** Cal Trigger Points */

		/* Set scales */
		m_nVPD_CH1 = pDoc->m_nCH1_VOLTS;
		m_nVPD_CH2 = pDoc->m_nCH2_VOLTS;
		m_nVPD_CH3 = pDoc->m_nCH3_VOLTS;
		m_nVPD_CH4 = pDoc->m_nCH4_VOLTS;

		/** Draw Traces */
		OnDrawCH1();
		OnDrawCH2();
		OnDrawCH3();
		OnDrawCH4();

		/* Enable Timer again */
		SetTimer(nTimerRefresh,m_nRefreshRate,NULL);	
	}
	CView::OnTimer(nIDEvent);
}

void CScopeView::OnInitialUpdate() 
{
	CView::OnInitialUpdate();
	pDoc =(CScopeDoc*)m_pDocument;
	pApp = (CScopeApp*)AfxGetApp();

	SetTimer(nTimerRefresh,m_nRefreshRate,NULL);	

	penGrid = new(CPen);
	penGrid->CreatePen(PS_SOLID,1,WHITE);
	penCH1 = new(CPen);
	penCH1->CreatePen(PS_SOLID,2,LIGHTRED);
	penCH2 = new(CPen);
	penCH2->CreatePen(PS_SOLID,2,LIGHTGREEN);
	penCH3 = new(CPen);
	penCH3->CreatePen(PS_SOLID,2,LIGHTBLUE);
	penCH4 = new(CPen);
	penCH4->CreatePen(PS_SOLID,2,YELLOW);

	m_ColorGridBackGround = BLACK;


	OnDrawGrid();
}

void CScopeView::OnDestroy() 
{
	CView::OnDestroy();
	



	KillTimer(nTimerRefresh);

}



void CScopeView::OnSize(UINT nType, int cx, int cy) 
{
	CView::OnSize(nType, cx, cy);

	m_nClientWidth = cx;
	m_nClientHeight = cy;

	m_ClientRect.bottom = 0;
	m_ClientRect.left = 0;
	m_ClientRect.right = m_nClientWidth;
	m_ClientRect.top = m_nClientHeight;

	PrepareTraces(); // Resize channel traces
	PrepareGrid();
	OnDrawGrid();
	OnDrawCH1();
	OnDrawCH2();
	OnDrawCH3();
	OnDrawCH4();
}


void CScopeView::OnGridConfiguration() 
{
	CGridConfigDlg dlg;
	dlg.m_X = x/10;
	dlg.m_Y = y/10;
	dlg.m_ColorGridBackGround = m_ColorGridBackGround;
	dlg.m_RefreshRate = m_nRefreshRate;
	dlg.m_nBufferSize = pApp->m_nBufferSize;

	if(dlg.DoModal() == IDOK)
	{
		x = dlg.m_X*10;
		y = dlg.m_Y*10;
		m_ColorGridBackGround = dlg.m_ColorGridBackGround;


		if(m_nCH1POS>x) m_nCH1POS=x;
		if(m_nCH2POS>x) m_nCH1POS=x;
		if(m_nCH3POS>x) m_nCH1POS=x;
		if(m_nCH4POS>x) m_nCH1POS=x;

		PrepareGrid();
		PrepareTraces();
		OnDrawGrid();
		OnDrawCH1();
		OnDrawCH2();
		OnDrawCH3();
		OnDrawCH4();
	

		m_nRefreshRate = dlg.m_RefreshRate;
		KillTimer(nTimerRefresh);	// Kill Refresh Timmer
		SetTimer(nTimerRefresh,m_nRefreshRate,NULL); // New Refresh Timer

		pApp->m_nBufferSize = dlg.m_nBufferSize;

		pDoc->UpdateAllViews(NULL);
	}

	
}

void CScopeView::PrepareGrid()
{
	double fW, fH;
	int n;

	/* Border */
	int nBD = 10;

	fW = ((double)(m_nClientWidth-(nBD*2)))/((double)x);
	fH = ((double)(m_nClientHeight-(nBD*2)))/((double)y);

	m_nGridX[0] = nBD;
	m_nGridY[0] = nBD;


	for (n = 1; n <=x;n++)
	{
		m_nGridX[n] = ((int)(fW * n))+nBD;
		
	}

	for (n = 1; n <=y;n++)
	{
		 m_nGridY[n] = ((int)(fH * n))+nBD;
	}
}

void CScopeView::OnHideShowSideBar() 
{
		CView* pView = (CView*)this;
		CChildFrame* pChild = (CChildFrame*)pView->GetParentFrame();
		pChild->HideShowPanel();
}



void CScopeView::OnPaint() 
{
	CPaintDC dc(this); // device context for painting
	CDC memDC;
	CBitmap memBitmap;
	CBitmap* oldBitmap; // bitmap originally found in CMemDC

	// no real plotting work is performed here, 
	// just putting the existing bitmaps on the client

	// to avoid flicker, establish a memory dc, draw to it 
	// and then BitBlt it to the client
	memDC.CreateCompatibleDC(&dc) ;
	memBitmap.CreateCompatibleBitmap(&dc, m_nClientWidth, m_nClientHeight);
	oldBitmap = (CBitmap *)memDC.SelectObject(&memBitmap);

	if (memDC.GetSafeHdc() != NULL)
	{
		// first drop the grid on the memory dc
		memDC.BitBlt(0, 0, m_nClientWidth, m_nClientHeight, 
					&m_dcGrid, 0, 0, SRCCOPY);
    
	
		// now add the plot on top as a "pattern" via SRCPAINT.
		// works well with dark background and a light plot

		/* Only paint channels thats are enabled as this will reduce CPU load */
		if (pDoc->m_bCH1_ENABLE == true) // Only draw if CH1 is enabled
		{
			memDC.BitBlt(0, 0, m_nClientWidth, m_nClientHeight, 
					&m_dcCH1, 0, 0, SRCPAINT);  //SRCPAINT
		}

		if (pDoc->m_bCH2_ENABLE == true) // Only draw if CH2 is enabled
		{
			memDC.BitBlt(0, 0, m_nClientWidth, m_nClientHeight, 
					&m_dcCH2, 0, 0, SRCPAINT);  //SRCPAINT
		}

		if (pDoc->m_bCH3_ENABLE == true) // Only draw if CH3 is enabled
		{
			memDC.BitBlt(0, 0, m_nClientWidth, m_nClientHeight, 
					&m_dcCH3, 0, 0, SRCPAINT);  //SRCPAINT
		}

		if (pDoc->m_bCH4_ENABLE == true) // Only draw if CH4 is enabled
		{
			memDC.BitBlt(0, 0, m_nClientWidth, m_nClientHeight, 
				&m_dcCH4, 0, 0, SRCPAINT);  //SRCPAINT
		}


		// finally send the result to the display
		dc.BitBlt(0, 0, m_nClientWidth, m_nClientHeight, 
				&memDC, 0, 0, SRCCOPY);

	}

	 memDC.SelectObject(oldBitmap);

}

void CScopeView::OnDrawGrid()
{

/* -----------------------------------------------------------
   | Fuction:     void CScopeView::OnDrawGrid()              |
   | Description: Draws grid in memory then updates display  |
   | Return:      Void.                                      |
   | Date:        09/12/2001                                 |
   | Verison:     1.4                                        |
   | By:          Colin McCord                               |
   ----------------------------------------------------------- */

	int n,i;
	CClientDC dc(this);
	CBitmap m_bitmapGrid;
	CBitmap *m_bitmapOldGrid = NULL;

    // if we don't have one yet, set up a memory dc for the grid
	if (m_dcGrid.GetSafeHdc() == NULL)	m_dcGrid.CreateCompatibleDC(&dc);

	m_bitmapGrid.CreateCompatibleBitmap(&dc, m_nClientWidth, m_nClientHeight);
	m_bitmapOldGrid = m_dcGrid.SelectObject(&m_bitmapGrid);
	m_bitmapOldGrid->DeleteObject(); // Make sure there is not a memory leak 

	/* If the pen is not setup, do not continue */
	if(penGrid == NULL) return;
	
	/* Select the grid pen for use with the DC */
	CPen* pOldPen = m_dcGrid.SelectObject(penGrid);

	CBrush Brush;
	Brush.CreateSolidBrush(m_ColorGridBackGround);	// Grid Back Color
	m_dcGrid.FillRect(&m_ClientRect,&Brush);

	/* Draw Border */
	m_dcGrid.MoveTo(m_nGridX[0],m_nGridY[0]);
	m_dcGrid.LineTo(m_nGridX[x],m_nGridY[0]);
	m_dcGrid.LineTo(m_nGridX[x],m_nGridY[y]);
	m_dcGrid.LineTo(m_nGridX[0],m_nGridY[y]);
	m_dcGrid.LineTo(m_nGridX[0],m_nGridY[0]);


	/* Draw V Gray Lines */
	for (n=10; n < x;n=n+10)
	{	
		for(i = m_nGridY[0]; i<m_nGridY[y];i++) m_dcGrid.SetPixel(m_nGridX[n],i,GRAY);
	}
	

	/* Draw H gray line */
	for (n=10; n < y; n=n+10)
	{
		for(i = m_nGridX[0]; i<m_nGridX[x];i++) m_dcGrid.SetPixel(i,m_nGridY[n],GRAY);
	}

	/* Draw Center Axis */
	m_dcGrid.MoveTo(m_nGridX[x/2],m_nGridY[0]);
	m_dcGrid.LineTo(m_nGridX[x/2],m_nGridY[y]);
	m_dcGrid.MoveTo(m_nGridX[0],m_nGridY[y/2]);
	m_dcGrid.LineTo(m_nGridX[x],m_nGridY[y/2]);

	/* Draw V Ticks on H Center */
	for (n=2; n < x; n=n+2)
	{
		m_dcGrid.MoveTo(m_nGridX[n],m_nGridY[(y/2)-1]);
		m_dcGrid.LineTo(m_nGridX[n],m_nGridY[(y/2)]);
	}

	/* Draw H Ticks on V Center */
	for (n=2; n < y; n=n+2)
	{
		m_dcGrid.MoveTo(m_nGridX[(x/2)],m_nGridY[n]);
		m_dcGrid.LineTo(m_nGridX[(x/2)+1],m_nGridY[n]);	
	}

	m_dcGrid.SelectObject(pOldPen);

	/* Update Display */
	Invalidate(TRUE);
}



BOOL CScopeView::Create(LPCTSTR lpszClassName, LPCTSTR lpszWindowName, DWORD dwStyle, const RECT& rect, CWnd* pParentWnd, UINT nID, CCreateContext* pContext) 
{
	/* Overrides the Normal Refesh and stops ficker */
	static CString className = AfxRegisterWndClass(CS_HREDRAW | CS_VREDRAW);

	return CWnd::Create(className, lpszWindowName, dwStyle, rect, pParentWnd, nID, pContext);
}

void CScopeView::OnLButtonDown(UINT nFlags, CPoint point) 
{
	CString temp;

	CRect CRectTEMP,CRectTEMP2;
	CRectTEMP.top =  m_nGridY[m_nCH1POS]-5;
	CRectTEMP.bottom = m_nGridY[m_nCH1POS]+5;
	CRectTEMP.left = 0;
	CRectTEMP.right = 10;

	CRectTEMP2.top =  0;
	CRectTEMP2.bottom = 10;
	CRectTEMP2.left = m_nGridX[x/2+m_nCH1_Offset]-5;
	CRectTEMP2.right = m_nGridX[x/2+m_nCH1_Offset]+5;
		
	
	if (pDoc->m_bCH1_ENABLE == true) // Check CH1 is enabled first
	{
		/* User has clicked on the trangle for positing CH1 */
		if (CRectTEMP.PtInRect(point))
		{
			m_bChangeZeroCH1=true;  // Set This flag
			CView::OnLButtonDown(nFlags, point);
			return;
		}


		/* User has clicked on the offset trangle */
		if (CRectTEMP2.PtInRect(point))
		{
			m_bChangeOffsetCH1=true;  // Set This flag
			CView::OnLButtonDown(nFlags, point);
			return;
		}


	}


	/** Channel 2 **/
	CRectTEMP.top =  m_nGridY[m_nCH2POS]-5;
	CRectTEMP.bottom = m_nGridY[m_nCH2POS]+5;
	CRectTEMP2.left = m_nGridX[x/2+m_nCH2_Offset]-5;
	CRectTEMP2.right = m_nGridX[x/2+m_nCH2_Offset]+5;
		
	if (pDoc->m_bCH2_ENABLE == true) // Check CH2 is enabled first
	{
		/* User has clicked on the trangle for positing CH2 */
		if (CRectTEMP.PtInRect(point))
		{
			m_bChangeZeroCH2=true;  // Set This flag
			CView::OnLButtonDown(nFlags, point);
			return;
		}

		/* User has clicked on the offset trangle */
		if (CRectTEMP2.PtInRect(point))
		{
			m_bChangeOffsetCH2=true;  // Set This flag
			CView::OnLButtonDown(nFlags, point);
			return;
		}

	}

	/** Channel 3 **/
	CRectTEMP.top =  m_nGridY[m_nCH3POS]-5;
	CRectTEMP.bottom = m_nGridY[m_nCH3POS]+5;
	CRectTEMP2.left = m_nGridX[x/2+m_nCH3_Offset]-5;
	CRectTEMP2.right = m_nGridX[x/2+m_nCH3_Offset]+5;
		
	if (pDoc->m_bCH3_ENABLE == true) // Check CH3 is enabled first
	{
		/* User has clicked on the trangle for positing CH3 */
		if (CRectTEMP.PtInRect(point))
		{
			m_bChangeZeroCH3=true;  // Set This flag
			CView::OnLButtonDown(nFlags, point);
			return;
		}

		/* User has clicked on the offset trangle */
		if (CRectTEMP2.PtInRect(point))
		{
			m_bChangeOffsetCH3=true;  // Set This flag
			CView::OnLButtonDown(nFlags, point);
			return;
		}

	}


	/** Channel 4 **/
	CRectTEMP.top =  m_nGridY[m_nCH4POS]-5;
	CRectTEMP.bottom = m_nGridY[m_nCH4POS]+5;
	CRectTEMP2.left = m_nGridX[x/2+m_nCH4_Offset]-5;
	CRectTEMP2.right = m_nGridX[x/2+m_nCH4_Offset]+5;
		
	if (pDoc->m_bCH4_ENABLE == true) // Check CH4 is enabled first
	{
		/* User has clicked on the trangle for positing CH3 */
		if (CRectTEMP.PtInRect(point))
		{
			m_bChangeZeroCH4=true;  // Set This flag
			CView::OnLButtonDown(nFlags, point);
			return;
		}

		/* User has clicked on the offset trangle */
		if (CRectTEMP2.PtInRect(point))
		{
			m_bChangeOffsetCH4=true;  // Set This flag
			CView::OnLButtonDown(nFlags, point);
			return;
		}

	}

	CView::OnLButtonDown(nFlags, point);
}

void CScopeView::OnLButtonUp(UINT nFlags, CPoint point) 
{
	// Left mouse button has been release hence make sure this flag is not set
	m_bChangeZeroCH1 = false; 
	m_bChangeZeroCH2 = false; 
	m_bChangeZeroCH3 = false; 
	m_bChangeZeroCH4 = false; 
	m_bChangeOffsetCH1=false;  
	m_bChangeOffsetCH2=false;  
	m_bChangeOffsetCH3=false;  
	m_bChangeOffsetCH4=false;  
	CView::OnLButtonUp(nFlags, point);
}

void CScopeView::OnMouseMove(UINT nFlags, CPoint point) 
{

/* ------------------------------------------------------------------------
   | Fuction:     void CScopeView::OnMouseMove(UINT nFlags, CPoint point) | 
   | Description: Sets the mouse cursor, and if flag is set               |
   |              moves CH1/CH2/CH3/CH4 zero point position with the mouse|
   |              display.                                                |
   | Return:      Void.                                                   |
   | Date:        15/03/2002                                              |
   | Verison:     1.6                                                     |
   | By:          Colin McCord                                            |
   ------------------------------------------------------------------------ */

	int i;

	/* load standard mouse cursor */
	SetCursor(AfxGetApp()->LoadStandardCursor(IDC_ARROW));

	CRect CRectTEMP;

	/* Move the zero point of CH1 */
	if (m_bChangeZeroCH1==true)
	{
		for (i = 0; i < y; i++)
		{
			if (m_nGridY[0] > point.y) break;
			if (m_nGridY[i] == point.y) break;
			else if (m_nGridY[i] < point.y && m_nGridY[i+1] > point.y) break;
		}
		m_nCH1POS = i;

		OnDrawCH1();
	}
	
	/* Move the zero point of CH2 */
	if (m_bChangeZeroCH2==true)
	{
		for (i = 0; i < y; i++)
		{
			if (m_nGridY[0] > point.y) break;
			if (m_nGridY[i] == point.y) break;
			else if (m_nGridY[i] < point.y && m_nGridY[i+1] > point.y) break;
		}
		m_nCH2POS = i;

		OnDrawCH2();
	}

	/* Move the zero point of CH3 */
	if (m_bChangeZeroCH3==true)
	{
		for (i = 0; i < y; i++)
		{
			if (m_nGridY[0] > point.y) break;
			if (m_nGridY[i] == point.y) break;
			else if (m_nGridY[i] < point.y && m_nGridY[i+1] > point.y) break;
		}
		m_nCH3POS = i;

		OnDrawCH3();
	}

	/* Move the zero point of CH4 */
	if (m_bChangeZeroCH4==true)
	{
		for (i = 0; i < y; i++)
		{
			if (m_nGridY[0] > point.y) break;
			if (m_nGridY[i] == point.y) break;
			else if (m_nGridY[i] < point.y && m_nGridY[i+1] > point.y) break;
		}
		m_nCH4POS = i;

		OnDrawCH4();
	}


	/* Move the Offset point of CH1 */
	if (m_bChangeOffsetCH1==true)
	{
		for (i = 0; i < x; i++)
		{
			if (m_nGridX[0] > point.x) break;
			if (m_nGridX[i] == point.x) break;
			else if (m_nGridX[i] < point.x && m_nGridX[i+1] > point.x) break;
		}
		m_nCH1_Offset = i-(x/2);

		OnDrawCH1();
	}

	/* Move the Offset point of CH2 */
	if (m_bChangeOffsetCH2==true)
	{
		for (i = 0; i < x; i++)
		{
			if (m_nGridX[0] > point.x) break;
			if (m_nGridX[i] == point.x) break;
			else if (m_nGridX[i] < point.x && m_nGridX[i+1] > point.x) break;
		}
		m_nCH2_Offset = i-(x/2);

		OnDrawCH2();
	}

	/* Move the Offset point of CH3 */
	if (m_bChangeOffsetCH3==true)
	{
		for (i = 0; i < x; i++)
		{
			if (m_nGridX[0] > point.x) break;
			if (m_nGridX[i] == point.x) break;
			else if (m_nGridX[i] < point.x && m_nGridX[i+1] > point.x) break;
		}
		m_nCH3_Offset = i-(x/2);

		OnDrawCH3();
	}

	/* Move the Offset point of CH4 */
	if (m_bChangeOffsetCH4==true)
	{
		for (i = 0; i < x; i++)
		{
			if (m_nGridX[0] > point.x) break;
			if (m_nGridX[i] == point.x) break;
			else if (m_nGridX[i] < point.x && m_nGridX[i+1] > point.x) break;
		}
		m_nCH4_Offset = i-(x/2);

		OnDrawCH4();
	}
	CView::OnMouseMove(nFlags, point);
}

void CScopeView::OnDrawCH1()
{
/* -----------------------------------------------------------
   | Fuction:     void CScopeView::OnDrawCH1()               |
   | Description: Draws CH1 wavefrom, in memory then updates |
   |              display.                                   |
   | Return:      Void.                                      |
   | Date:        04/04/2002                                 |
   | Verison:     1.9                                        |
   | By:          Colin McCord                               |
   ----------------------------------------------------------- */

	CClientDC dc(this);
	int i,n,y1,y2,a=1,m_Skip, m_MUX;
	CBrush Brush;					
	Brush.CreateSolidBrush(BLACK);  // Set brush colour to black
	CRect CRectTEMP;

	// Channel 1 Member DC not setup yet
	if (m_dcCH1.GetSafeHdc() == NULL) 
	{
		PrepareTraces();
	}
	

	/* If channel 1 pen is not created yet, do not contiune */
	if (penCH1 == NULL) return;

	/* Select CH1 pen */
	CPen* pOldPen = m_dcCH1.SelectObject(penCH1);

	/* Clear display*/
	m_dcCH1.SetBkColor(BLACK);
	m_dcCH1.FillRect(&m_ClientRect,&Brush);

	/* If Channel 1 is disabled do not continue */
	if (pDoc->m_bCH1_ENABLE == false) 
	{
		m_dcCH1.SelectObject(pOldPen);
		Invalidate(TRUE); // Redraw Display
		return;
	}


	if (pDoc->m_TimeBaseMUX < 0)	// Increased Timebase
	{
		m_Skip = pDoc->m_TimeBaseMUX * -1;
		m_MUX  = 1;
	}
	else							// Reduced Timebase
	{
		m_Skip = 1;
		m_MUX = pDoc->m_TimeBaseMUX;
	}

	/*
	CString temp2;
	temp2.Format("m_MUX = %d, Skip = %d", m_MUX,m_Skip);
	m_dcCH1.SetTextColor(LIGHTGREEN);
	m_dcCH1.TextOut(10,10,temp2);*/


	for(i = m_nTriggerCH1-(m_nCH1_Offset*m_MUX/m_Skip);i <= 10000; i= i+m_MUX) /* Draw waveform to memory DC */
	{	
			if(m_Skip >5000) break;	// Check for possible overflow

			y1 = m_nCH1POS+pApp->m_nCH1Volt[i-m_MUX]/m_nVPD_CH1;
			if (y1 > y) y1 = y;
			if (y1 < 0) y1 = 0;
			if (a-m_Skip<0) a= m_Skip;
			if(i >=0) m_dcCH1.MoveTo(m_nGridX[a-m_Skip],m_nGridY[y1]);
		
			y2 = m_nCH1POS+pApp->m_nCH1Volt[i]/m_nVPD_CH1;
			if (y2 > y) y2 = y;
			if (y2 < 0) y2 = 0;
			if(i >=0) m_dcCH1.LineTo(m_nGridX[a],m_nGridY[y2]);

			a = a + m_Skip;

			if ( a > x) 
			{
				i = 10001;
				break;
			}
	}

	/***  draw zero point Arrow */
	m_dcCH1.MoveTo(8,m_nGridY[m_nCH1POS]);
	m_dcCH1.LineTo(2,m_nGridY[m_nCH1POS]-5);
	m_dcCH1.LineTo(2,m_nGridY[m_nCH1POS]+5);
	m_dcCH1.LineTo(8,m_nGridY[m_nCH1POS]);

	/* Draw Zero point dotted line */
	for(i = 10; i<m_nGridX[x]-3;i+=6)
	{
		for(n=0;n<=3;n++)	m_dcCH1.SetPixel(i+n,m_nGridY[m_nCH1POS],LIGHTRED);
	}

	/*** draw offset Arrow */
	m_dcCH1.MoveTo(m_nGridX[x/2+m_nCH1_Offset],8);
	m_dcCH1.LineTo(m_nGridX[x/2+m_nCH1_Offset]-5,2);
	m_dcCH1.LineTo(m_nGridX[x/2+m_nCH1_Offset]+5,2);
	m_dcCH1.LineTo(m_nGridX[x/2+m_nCH1_Offset],8);

	m_dcCH1.SelectObject(pOldPen);
	Invalidate(TRUE); // Redraw Display
}



void CScopeView::PrepareTraces()
{
/* -----------------------------------------------------------
   | Fuction:     void CScopeView::PrepareTraces()           |
   | Description: Creates trace memory DCs and bitmaps.      | 
   | Return:      Void.                                      |
   | Date:        15/03/2002                                 |
   | Verison:     1.1                                        |
   | By:          Colin McCord                               |
   ----------------------------------------------------------- */

	CClientDC dc(this);	// Screen DC.

	/*** Channel 1 ****/
	CBitmap *m_bitmapOldCH1 = NULL;	// Pointer to old display bitmap
				
	m_bitmapCH1 = new CBitmap;	// Create a new bitmap

	/** Create new bitmap and delete old, (Channel 1) **/
	if (m_dcCH1.GetSafeHdc() == NULL) m_dcCH1.CreateCompatibleDC(&dc);

	m_bitmapCH1->CreateCompatibleBitmap(&dc, m_nClientWidth, m_nClientHeight);
	m_bitmapOldCH1 = m_dcCH1.SelectObject(m_bitmapCH1);
	m_bitmapOldCH1->DeleteObject(); // Make Sure there is not a memory leak 


	/*** Channel 2 ****/
	CBitmap *m_bitmapOldCH2 = NULL;	// Pointer to old display bitmap
	m_bitmapCH2 = new CBitmap;	// Create a new bitmap

	/** Create new bitmap and delete old, (Channel 2) **/
	if (m_dcCH2.GetSafeHdc() == NULL) m_dcCH2.CreateCompatibleDC(&dc);

	m_bitmapCH2->CreateCompatibleBitmap(&dc, m_nClientWidth, m_nClientHeight);
	m_bitmapOldCH2 = m_dcCH2.SelectObject(m_bitmapCH2);
	m_bitmapOldCH2->DeleteObject(); // Make Sure there is not a memory leak 


	/*** Channel 3 ****/
	CBitmap *m_bitmapOldCH3 = NULL;	// Pointer to old display bitmap
	m_bitmapCH3 = new CBitmap;	// Create a new bitmap

	/** Create new bitmap and delete old, (Channel 3) **/
	if (m_dcCH3.GetSafeHdc() == NULL) m_dcCH3.CreateCompatibleDC(&dc);

	m_bitmapCH3->CreateCompatibleBitmap(&dc, m_nClientWidth, m_nClientHeight);
	m_bitmapOldCH3 = m_dcCH3.SelectObject(m_bitmapCH3);
	m_bitmapOldCH3->DeleteObject(); // Make Sure there is not a memory leak 


	/*** Channel 4 ****/
	CBitmap *m_bitmapOldCH4 = NULL;	// Pointer to old display bitmap
	m_bitmapCH4 = new CBitmap;	// Create a new bitmap

	/** Create new bitmap and delete old, (Channel 4) **/
	if (m_dcCH4.GetSafeHdc() == NULL) m_dcCH4.CreateCompatibleDC(&dc);

	m_bitmapCH4->CreateCompatibleBitmap(&dc, m_nClientWidth, m_nClientHeight);
	m_bitmapOldCH4 = m_dcCH4.SelectObject(m_bitmapCH4);
	m_bitmapOldCH4->DeleteObject(); // Make Sure there is not a memory leak 
}

void CScopeView::OnEditCopy() 
{

/* -----------------------------------------------------------
   | Fuction:     void CScopeView::OnEditCopy()              |
   | Description: Copys the scope display to the Windows     |
   |              clip board.                                | 
   | Return:      Void.                                      |
   | Date:        15/02/2002                                 |
   | Verison:     1.0                                        |
   | By:          Colin McCord                               |
   ----------------------------------------------------------- */

	CRect		rect;
	CClientDC	dc(this);
	CDC 		memDC;
	CBitmap		bitmap;
	
	GetClientRect(&rect); 

	// Create memDC
	memDC.CreateCompatibleDC(&dc);
	bitmap.CreateCompatibleBitmap(&dc, rect.Width(), rect.Height());	
	CBitmap* pOldBitmap = memDC.SelectObject(&bitmap);

	// Fill in memDC
	memDC.FillSolidRect(rect, dc.GetBkColor()); 


	// Redraw grid to memory DC
	if (memDC.GetSafeHdc() != NULL)
	{
		// first drop the grid on the memory dc
		memDC.BitBlt(0, 0, m_nClientWidth, m_nClientHeight, 
					&m_dcGrid, 0, 0, SRCCOPY);
    
		// now add the plot on top as a "pattern" via SRCPAINT.
		// works well with dark background and a light plot
		memDC.BitBlt(0, 0, m_nClientWidth, m_nClientHeight, 
				&m_dcCH1, 0, 0, SRCPAINT);  //SRCPAINT

		memDC.BitBlt(0, 0, m_nClientWidth, m_nClientHeight, 
				&m_dcCH2, 0, 0, SRCPAINT);  //SRCPAINT

		memDC.BitBlt(0, 0, m_nClientWidth, m_nClientHeight, 
				&m_dcCH3, 0, 0, SRCPAINT);  //SRCPAINT
	
		memDC.BitBlt(0, 0, m_nClientWidth, m_nClientHeight, 
				&m_dcCH4, 0, 0, SRCPAINT);  //SRCPAINT
	}


	// Copy contents of memDC to clipboard
	OpenClipboard();	
	EmptyClipboard();
	SetClipboardData(CF_BITMAP, bitmap.GetSafeHandle());	
	CloseClipboard();

	// Clean up
	memDC.SelectObject(pOldBitmap);	
	bitmap.Detach();	
}

void CScopeView::OnFileExport()
{
/* -----------------------------------------------------------
   | Fuction:     void CScopeView::OnFileExport()            |
   | Description: User specifies filename using CFileDialog, |
   |              then the scope display is save to that     |
   |			  filename using the standard bitmap         |
   |			  grahpic format.                            |
   | Return:      Void.                                      |
   | Date:        16/02/2002                                 |
   | Verison:     1.1                                        |
   | By:          Colin K McCord                             |
   ----------------------------------------------------------- */


	HANDLE		hDIB;	
	CRect		rect;
	CClientDC	dc(this);
	CDC 		memDC;
	CBitmap		bitmap;
	
	GetClientRect(&rect); 

	// Create memDC
	memDC.CreateCompatibleDC(&dc);
	bitmap.CreateCompatibleBitmap(&dc, rect.Width(), rect.Height());	
	CBitmap* pOldBitmap = memDC.SelectObject(&bitmap);

	// Fill in memDC
	memDC.FillSolidRect(rect, dc.GetBkColor()); 
	

	// Redraw grid to memory DC
	if (memDC.GetSafeHdc() != NULL)
	{
		// first drop the grid on the memory dc
		memDC.BitBlt(0, 0, m_nClientWidth, m_nClientHeight, 
					&m_dcGrid, 0, 0, SRCCOPY);
    
		// now add the plot on top as a "pattern" via SRCPAINT.
		// works well with dark background and a light plot
		memDC.BitBlt(0, 0, m_nClientWidth, m_nClientHeight, 
				&m_dcCH1, 0, 0, SRCPAINT);  //SRCPAINT
	
		memDC.BitBlt(0, 0, m_nClientWidth, m_nClientHeight, 
				&m_dcCH2, 0, 0, SRCPAINT);  //SRCPAINT

		memDC.BitBlt(0, 0, m_nClientWidth, m_nClientHeight, 
				&m_dcCH3, 0, 0, SRCPAINT);  //SRCPAINT
	
		memDC.BitBlt(0, 0, m_nClientWidth, m_nClientHeight, 
				&m_dcCH4, 0, 0, SRCPAINT);  //SRCPAINT


	}

	
	/* Convert Device-Dependent Bitmap (DDB) to Device-independent Bitmap (DIB) */
	hDIB = DDBToDIB(bitmap, BI_RGB, NULL);

	CFileDialog dlg (FALSE, _T("*.bmp"), _T("Scope"), OFN_HIDEREADONLY,_T("Windows or OS/2 Bitmap (*.bmp)|*.bmp||"));
	dlg.m_ofn.lpstrTitle = "Export Scope Display";
	if (IDOK == dlg.DoModal())
	{
	
		/* Check for existence */
		if( (_access(dlg.GetPathName(), 0 )) != -1 )
		{
			if(MessageBox("File '" + dlg.GetPathName() + "' Already exists. Do you want to replace the existing file ?","Export Scope Display", MB_YESNO | MB_ICONINFORMATION) != IDYES)
				return;
		}
	
		WriteDIB(dlg.GetPathName(),hDIB);	// Save as Bitmap
	
	}	

}

HANDLE CScopeView::DDBToDIB(CBitmap &bitmap, DWORD dwCompression, CPalette *pPal)
{
/* -----------------------------------------------------------
   | Fuction:     HANDLE CScopeView::DDBToDIB(...)           |
   | Description: Convert Device-Dependent Bitmap (DDB) to   |
   |              Device-independent Bitmap (DIB).           |
   | Return:      Void.                                      |
   | Date:        16/02/2002                                 |
   | Verison:     1.3                                        |
   | By:          Colin K McCord                             |
   ----------------------------------------------------------- */
 
	// DDBToDIB		- Creates a DIB from a DDB
	// bitmap		- Device dependent bitmap
	// dwCompression- Type of compression - see BITMAPINFOHEADER
	// pPal			- Logical palette


	BITMAP				bm;
	BITMAPINFOHEADER	bi;
	LPBITMAPINFOHEADER 	lpbi;
	DWORD				dwLen;
	HANDLE				hDIB;
	HANDLE				handle;
	HDC 				hDC;
	HPALETTE			hPal;


	ASSERT( bitmap.GetSafeHandle() );

	// The function has no arg for bitfields
	if( dwCompression == BI_BITFIELDS )
		return NULL;

	// If a palette has not been supplied use defaul palette
	hPal = (HPALETTE) pPal->GetSafeHandle();
	if (hPal==NULL)
		hPal = (HPALETTE) GetStockObject(DEFAULT_PALETTE);

	// Get bitmap information
	bitmap.GetObject(sizeof(bm),(LPSTR)&bm);

	// Initialize the bitmapinfoheader
	bi.biSize			= sizeof(BITMAPINFOHEADER);
	bi.biWidth			= bm.bmWidth;
	bi.biHeight 		= bm.bmHeight;
	bi.biPlanes 		= 1;
	bi.biBitCount		= bm.bmPlanes * bm.bmBitsPixel;
	bi.biCompression	= dwCompression;
	bi.biSizeImage		= 0;
	bi.biXPelsPerMeter	= 0;
	bi.biYPelsPerMeter	= 0;
	bi.biClrUsed		= 0;
	bi.biClrImportant	= 0;

	// Compute the size of the  infoheader and the color table
	int nColors = (1 << bi.biBitCount);

	dwLen  = bi.biSize + nColors * sizeof(RGBQUAD);

	// We need a device context to get the DIB from
	hDC = ::GetDC(NULL);

	hPal = SelectPalette(hDC,hPal,FALSE);
	RealizePalette(hDC);

	// Allocate enough memory to hold bitmapinfoheader and color table
	hDIB = GlobalAlloc(GMEM_FIXED,dwLen);

	if (!hDIB){
		SelectPalette(hDC,hPal,FALSE);
		::ReleaseDC(NULL,hDC);
		return NULL;
	}

	lpbi = (LPBITMAPINFOHEADER)hDIB;

	*lpbi = bi;

	// Call GetDIBits with a NULL lpBits param, so the device driver 
	// will calculate the biSizeImage field 
	GetDIBits(hDC, (HBITMAP)bitmap.GetSafeHandle(), 0L, (DWORD)bi.biHeight,
			(LPBYTE)NULL, (LPBITMAPINFO)lpbi, (DWORD)DIB_RGB_COLORS);

	bi = *lpbi;

	// If the driver did not fill in the biSizeImage field, then compute it
	// Each scan line of the image is aligned on a DWORD (32bit) boundary
	if (bi.biSizeImage == 0){
		bi.biSizeImage = ((((bi.biWidth * bi.biBitCount) + 31) & ~31) / 8) 
						* bi.biHeight;

		// If a compression scheme is used the result may infact be larger
		// Increase the size to account for this.
		if (dwCompression != BI_RGB)
			bi.biSizeImage = (bi.biSizeImage * 3) / 2;
	}

	// Realloc the buffer so that it can hold all the bits
	dwLen += bi.biSizeImage;
	if (handle = GlobalReAlloc(hDIB, dwLen, GMEM_MOVEABLE))
		hDIB = handle;
	else{
		GlobalFree(hDIB);

		// Reselect the original palette
		SelectPalette(hDC,hPal,FALSE);
			::ReleaseDC(NULL,hDC);
		return NULL;
	}

	// Get the bitmap bits
	lpbi = (LPBITMAPINFOHEADER)hDIB;

	// FINALLY get the DIB
	BOOL bGotBits = GetDIBits( hDC, (HBITMAP)bitmap.GetSafeHandle(),
				0L,						// Start scan line
				(DWORD)bi.biHeight,		// # of scan lines
				(LPBYTE)lpbi 			// address for bitmap bits
				+ (bi.biSize + nColors * sizeof(RGBQUAD)),
				(LPBITMAPINFO)lpbi,		// address of bitmapinfo
				(DWORD)DIB_RGB_COLORS);	// Use RGB for color table

	if( !bGotBits )
	{
		GlobalFree(hDIB);
		SelectPalette(hDC,hPal,FALSE);
		::ReleaseDC(NULL,hDC);
		return NULL;
	}

	SelectPalette(hDC,hPal,FALSE);
	::ReleaseDC(NULL,hDC);
	return hDIB;
}

BOOL CScopeView::WriteDIB(CString szFile, HANDLE hDIB)
{
/* -----------------------------------------------------------
   | Fuction:     BOOL CScopeView::WriteDIB(........)        |
   | Description: Writes bitmap to specified file.           |
   | Return:      Void.                                      |
   | Date:        16/02/2002                                 |
   | Verison:     1.1                                        |
   | By:          Colin K McCord                             |
   ----------------------------------------------------------- */

	// WriteDIB		- Writes a DIB to file
	// Returns		- TRUE on success
	// szFile		- Name of file to write to
	// hDIB			- Handle of the DIB

	BITMAPFILEHEADER	hdr;
	LPBITMAPINFOHEADER	lpbi;

	if (!hDIB)
		return FALSE;

	CFile file;
	if( !file.Open( szFile, CFile::modeWrite|CFile::modeCreate) )
		return FALSE;

	lpbi = (LPBITMAPINFOHEADER)hDIB;

	int nColors = 1 << lpbi->biBitCount;

	// Fill in the fields of the file header 
	hdr.bfType		= ((WORD) ('M' << 8) | 'B');	// is always "BM"
	hdr.bfSize		= GlobalSize (hDIB) + sizeof( hdr );
	hdr.bfReserved1 	= 0;
	hdr.bfReserved2 	= 0;
	hdr.bfOffBits		= (DWORD) (sizeof( hdr ) + lpbi->biSize +
						nColors * sizeof(RGBQUAD));

	// Write the file header 
	file.Write( &hdr, sizeof(hdr) );

	// Write the DIB header and the bits 
	file.Write( lpbi, GlobalSize(hDIB) );

	return TRUE;

}

void CScopeView::OnPrint(CDC* pDC, CPrintInfo* pInfo) 
{
	CString Date;
	CTime t = CTime::GetCurrentTime();	
	Date = t.Format("%A, %B %d, %Y");
	TEXTMETRIC tm;
	pDC->GetTextMetrics(&tm);
	int cyChar = tm.tmHeight;
	CRect rectPage = pInfo->m_rectDraw;
	CFont* pOldFont = NULL;

	/* Get Border detail from Registry */
	int m_Top = pApp->GetProfileInt("PageSetup","Top",-1);
	int m_Bottom = pApp->GetProfileInt("PageSetup","Bottom",-1) * -1;
	int m_Left = pApp->GetProfileInt("PageSetup","Left",-1);
	int m_Right = pApp->GetProfileInt("PageSetup","Right",-1) * -1;
	if(m_Top == -1) m_Top = 100;
	if(m_Bottom == 1) m_Bottom = -100;
	if(m_Left == -1) m_Left = 0;
	if(m_Right == 1) m_Right = 0;

	/** Change font **/
	pOldFont = ChangeFont(pDC,130,FF_ROMAN, FW_BOLD);

	// Header
	pDC->SetTextAlign(TA_CENTER);
	pDC->TextOut(rectPage.right/2 + ((m_Left+m_Right)/2), rectPage.top+15+m_Top, "Scope: Put Title Here!");

	rectPage.top += cyChar + cyChar / 4;
	pDC->MoveTo(rectPage.left+200+m_Left, rectPage.top+50+m_Top);
	pDC->LineTo(rectPage.right-200+m_Right, rectPage.top+50+m_Top);
	rectPage.top += cyChar / 4;

	/* ** Restore default Font **/
	if (pOldFont != NULL) pDC->SelectObject(pOldFont);

	/** Change font **/
	pOldFont = ChangeFont(pDC,80,FF_ROMAN, FW_NORMAL);

	// Footer
	rectPage.bottom -= cyChar;
	pDC->SetTextAlign(TA_LEFT);
	pDC->TextOut(rectPage.left+200+m_Left, rectPage.bottom+m_Bottom, Date);
	pDC->SetTextAlign(TA_CENTER);
	if(	pInfo->m_nCurPage == 1) pDC->TextOut(rectPage.right/2 + ((m_Left+m_Right)/2), rectPage.bottom+m_Bottom, "Page 1");

	pDC->SetTextAlign(TA_RIGHT);
	pDC->TextOut(rectPage.right-200+m_Right, rectPage.bottom+m_Bottom, "By Colin K McCord");
	rectPage.bottom -= cyChar / 4;
	pDC->MoveTo(rectPage.left+200+m_Left, rectPage.bottom+m_Bottom);
	pDC->LineTo(rectPage.right-200+m_Right, rectPage.bottom+m_Bottom);
	rectPage.bottom -= cyChar / 4;

	/** Restore default Font **/
	if (pOldFont != NULL) pDC->SelectObject(pOldFont);


	// TODO: Draw Grid and traces 


	CView::OnPrint(pDC, pInfo);
}

void CScopeView::OnPrepareDC(CDC* pDC, CPrintInfo* pInfo) 
{
	pDC->SetMapMode(MM_ANISOTROPIC);

	CSize sizeDoc;
	sizeDoc.cx = 4672;
	sizeDoc.cy = 6774;
	sizeDoc.cy = -sizeDoc.cy;
	pDC->SetWindowExt(sizeDoc);

	CSize sizeNum, sizeDenom;
	sizeNum.cx = 1;
	sizeNum.cy = 1;
	sizeDenom.cx = 6;
	sizeDenom.cy = -6;
	
	int xLogPixPerInch = pDC->GetDeviceCaps(LOGPIXELSX);
	int yLogPixPerInch = pDC->GetDeviceCaps(LOGPIXELSY);

	long xExt = (long)sizeDoc.cx * xLogPixPerInch * sizeNum.cx;
	xExt /= 100 * (long)sizeDenom.cx;
	long yExt = (long)sizeDoc.cy * yLogPixPerInch * sizeNum.cy;
	yExt /= 100 * (long)sizeDenom.cy;

	pDC->SetViewportExt((int)xExt, (int)-yExt);	
	CView::OnPrepareDC(pDC, pInfo);
}

CFont* CScopeView::ChangeFont(CDC *pDC, long Height, BYTE PitchAndFamily, long Weight)
{
	LOGFONT logFont;
	memset(&logFont, 0, sizeof(LOGFONT));
	logFont.lfHeight = Height;
	logFont.lfPitchAndFamily = PitchAndFamily;
	logFont.lfWeight = Weight;
	CFont font;
	CFont* pOldFont = NULL;
	if (font.CreateFontIndirect(&logFont))
		pOldFont = pDC->SelectObject(&font);

	return pOldFont;
}

void CScopeView::OnDrawCH2()
{

/* -----------------------------------------------------------
   | Fuction:     void CScopeView::OnDrawCH2()               |
   | Description: Draws CH2 wavefrom, in memory then updates |
   |              display.                                   |
   | Return:      Void.                                      |
   | Date:        04/04/2002                                 |
   | Verison:     1.9                                        |
   | By:          Colin McCord                               |
   ----------------------------------------------------------- */

	CClientDC dc(this);
	int i,n,y1,y2,a=1,m_Skip, m_MUX;

	CBrush Brush;					
	Brush.CreateSolidBrush(BLACK);  // Set brush colour to black
	CRect CRectTEMP;


	// Channel 2 Member DC not setup yet
	if (m_dcCH2.GetSafeHdc() == NULL) 
	{
		PrepareTraces();
	}
	


	/* If channel 2pen is not created yet, do not contiune */
	if (penCH2 == NULL) return;


	/* Select CH2 pen */
	CPen* pOldPen = m_dcCH2.SelectObject(penCH2);


	/* Clear display*/
	m_dcCH2.SetBkColor(BLACK);
	m_dcCH2.FillRect(&m_ClientRect,&Brush);


	/* If Channel 2 is disabled do not continue */
	if (pDoc->m_bCH2_ENABLE == false) 
	{
		m_dcCH1.SelectObject(pOldPen);
		Invalidate(TRUE); // Redraw Display
		return;
	}

	if (pDoc->m_TimeBaseMUX < 0)	// Increased Timebase
	{
		m_Skip = pDoc->m_TimeBaseMUX * -1;
		m_MUX  = 1;
	}
	else							// Reduced Timebase
	{
		m_Skip = 1;
		m_MUX = pDoc->m_TimeBaseMUX;
	}

	for(i = m_nTriggerCH2-(m_nCH2_Offset*m_MUX/m_Skip);i <= 10000; i= i+m_MUX) /* Draw waveform to memory DC */
	{	
			if(m_Skip >5000) break;	// Check for possible overflow

			y1 = m_nCH2POS+pApp->m_nCH2Volt[i-m_MUX]/m_nVPD_CH2;
			if (y1 > y) y1 = y;
			if (y1 < 0) y1 = 0;
			if (a-m_Skip<0) a= m_Skip;
			if(i >=0) m_dcCH2.MoveTo(m_nGridX[a-m_Skip],m_nGridY[y1]);
		
			y2 = m_nCH2POS+pApp->m_nCH2Volt[i]/m_nVPD_CH2;
			if (y2 > y) y2 = y;
			if (y2 < 0) y2 = 0;
			if(i >=0) m_dcCH2.LineTo(m_nGridX[a],m_nGridY[y2]);
			a = a + m_Skip;

			if ( a > x) 
			{
				i = 10001;
				break;
			}
	}


	/***  draw zero point Arrow */
	m_dcCH2.MoveTo(8,m_nGridY[m_nCH2POS]);
	m_dcCH2.LineTo(2,m_nGridY[m_nCH2POS]-5);
	m_dcCH2.LineTo(2,m_nGridY[m_nCH2POS]+5);
	m_dcCH2.LineTo(8,m_nGridY[m_nCH2POS]);

	/* Draw Zero point dotted line */
	for(i = 10; i<m_nGridX[x]-3;i+=6)
	{
		for(n=0;n<=3;n++)	m_dcCH2.SetPixel(i+n,m_nGridY[m_nCH2POS],LIGHTGREEN);
	}

	/*** draw offset Arrow */
	m_dcCH2.MoveTo(m_nGridX[x/2+m_nCH2_Offset],8);
	m_dcCH2.LineTo(m_nGridX[x/2+m_nCH2_Offset]-5,2);
	m_dcCH2.LineTo(m_nGridX[x/2+m_nCH2_Offset]+5,2);
	m_dcCH2.LineTo(m_nGridX[x/2+m_nCH2_Offset],8);


	m_dcCH2.SelectObject(pOldPen);
	Invalidate(TRUE); // Redraw Display
}

void CScopeView::OnDrawCH3()
{

/* -----------------------------------------------------------
   | Fuction:     void CScopeView::OnDrawCH3()               |
   | Description: Draws CH3 wavefrom, in memory then updates |
   |              display.                                   |
   | Return:      Void.                                      |
   | Date:        04/04/2002                                 |
   | Verison:     1.9                                        |
   | By:          Colin McCord                               |
   ----------------------------------------------------------- */

	CClientDC dc(this);
	int i,n,y1,y2,a=1,m_Skip, m_MUX;

	CBrush Brush;					
	Brush.CreateSolidBrush(BLACK);  // Set brush colour to black
	CRect CRectTEMP;


	// Channel 3 Member DC not setup yet
	if (m_dcCH3.GetSafeHdc() == NULL) 
	{
		PrepareTraces();
	}
	


	/* If channel 3 pen is not created yet, do not contiune */
	if (penCH3 == NULL) return;


	/* Select CH3 pen */
	CPen* pOldPen = m_dcCH3.SelectObject(penCH3);


	/* Clear display*/
	m_dcCH3.SetBkColor(BLACK);
	m_dcCH3.FillRect(&m_ClientRect,&Brush);


	/* If Channel 3 is disabled do not continue */
	if (pDoc->m_bCH3_ENABLE == false) 
	{
		m_dcCH3.SelectObject(pOldPen);
		Invalidate(TRUE); // Redraw Display
		return;
	}



	if (pDoc->m_TimeBaseMUX < 0)	// Increased Timebase
	{
		m_Skip = pDoc->m_TimeBaseMUX * -1;
		m_MUX  = 1;
	}
	else							// Reduced Timebase
	{
		m_Skip = 1;
		m_MUX = pDoc->m_TimeBaseMUX;
	}

	for(i = m_nTriggerCH3-(m_nCH3_Offset*m_MUX/m_Skip);i <= 10000; i= i+m_MUX) /* Draw waveform to memory DC */
	{	
			if(m_Skip >5000) break;	// Check for possible overflow

			y1 = m_nCH3POS+pApp->m_nCH3Volt[i-m_MUX]/m_nVPD_CH3;
			if (y1 > y) y1 = y;
			if (y1 < 0) y1 = 0;
			if (a-m_Skip<0) a= m_Skip;
			if(i >=0) m_dcCH3.MoveTo(m_nGridX[a-m_Skip],m_nGridY[y1]);
		
			y2 = m_nCH3POS+pApp->m_nCH3Volt[i]/m_nVPD_CH3;
			if (y2 > y) y2 = y;
			if (y2 < 0) y2 = 0;
			if(i >=0) m_dcCH3.LineTo(m_nGridX[a],m_nGridY[y2]);
			a = a + m_Skip;

			if ( a > x) 
			{
				i = 10001;
				break;
			}
	}


	/***  draw zero point Arrow */
	m_dcCH3.MoveTo(8,m_nGridY[m_nCH3POS]);
	m_dcCH3.LineTo(2,m_nGridY[m_nCH3POS]-5);
	m_dcCH3.LineTo(2,m_nGridY[m_nCH3POS]+5);
	m_dcCH3.LineTo(8,m_nGridY[m_nCH3POS]);

	/* Draw Zero point dotted line */
	for(i = 10; i<m_nGridX[x]-3;i+=6)
	{
		for(n=0;n<=3;n++)	m_dcCH3.SetPixel(i+n,m_nGridY[m_nCH3POS],LIGHTBLUE);
	}


	/*** draw offset Arrow */
	m_dcCH3.MoveTo(m_nGridX[x/2+m_nCH3_Offset],8);
	m_dcCH3.LineTo(m_nGridX[x/2+m_nCH3_Offset]-5,2);
	m_dcCH3.LineTo(m_nGridX[x/2+m_nCH3_Offset]+5,2);
	m_dcCH3.LineTo(m_nGridX[x/2+m_nCH3_Offset],8);

	m_dcCH3.SelectObject(pOldPen);
	Invalidate(TRUE); // Redraw Display
}

void CScopeView::OnDrawCH4()
{

/* -----------------------------------------------------------
   | Fuction:     void CScopeView::OnDrawCH3()               |
   | Description: Draws CH4 wavefrom, in memory then updates |
   |              display.                                   |
   | Return:      Void.                                      |
   | Date:        04/04/2002                                 |
   | Verison:     1.9                                        |
   | By:          Colin McCord                               |
   ----------------------------------------------------------- */

	CClientDC dc(this);
	int i,n,y1,y2,a=1,m_Skip, m_MUX;

	CBrush Brush;					
	Brush.CreateSolidBrush(BLACK);  // Set brush colour to black
	CRect CRectTEMP;


	// Channel 4 Member DC not setup yet
	if (m_dcCH4.GetSafeHdc() == NULL) 
	{
		PrepareTraces();
	}
	

	/* If channel 4 pen is not created yet, do not contiune */
	if (penCH4 == NULL) return;


	/* Select CH4 pen */
	CPen* pOldPen = m_dcCH4.SelectObject(penCH4);


	/* Clear display*/
	m_dcCH4.SetBkColor(BLACK);
	m_dcCH4.FillRect(&m_ClientRect,&Brush);


	/* If Channel 4 is disabled do not continue */
	if (pDoc->m_bCH4_ENABLE == false) 
	{
		m_dcCH4.SelectObject(pOldPen);
		Invalidate(TRUE); // Redraw Display
		return;
	}


	if (pDoc->m_TimeBaseMUX < 0)	// Increased Timebase
	{
		m_Skip = pDoc->m_TimeBaseMUX * -1;
		m_MUX  = 1;
	}
	else							// Reduced Timebase
	{
		m_Skip = 1;
		m_MUX = pDoc->m_TimeBaseMUX;
	}

	for(i = m_nTriggerCH4-(m_nCH4_Offset*m_MUX/m_Skip);i <= 10000; i= i+m_MUX) /* Draw waveform to memory DC */
	{	
			if(m_Skip >5000) break;	// Check for possible overflow

			y1 = m_nCH4POS+pApp->m_nCH4Volt[i-m_MUX]/m_nVPD_CH4;
			if (y1 > y) y1 = y;
			if (y1 < 0) y1 = 0;
			if (a-m_Skip<0) a= m_Skip;
			if(i >=0) m_dcCH4.MoveTo(m_nGridX[a-m_Skip],m_nGridY[y1]);
		
			y2 = m_nCH4POS+pApp->m_nCH4Volt[i]/m_nVPD_CH4;
			if (y2 > y) y2 = y;
			if (y2 < 0) y2 = 0;
			if(i >=0) m_dcCH4.LineTo(m_nGridX[a],m_nGridY[y2]);
			a = a + m_Skip;

			if ( a > x) 
			{
				i = 10001;
				break;
			}
	}


	/***  draw zero point Arrow */
	m_dcCH4.MoveTo(8,m_nGridY[m_nCH4POS]);
	m_dcCH4.LineTo(2,m_nGridY[m_nCH4POS]-5);
	m_dcCH4.LineTo(2,m_nGridY[m_nCH4POS]+5);
	m_dcCH4.LineTo(8,m_nGridY[m_nCH4POS]);

	/* Draw Zero point dotted line */
	for(i = 10; i<m_nGridX[x]-3;i+=6)
	{
		for(n=0;n<=3;n++)	m_dcCH4.SetPixel(i+n,m_nGridY[m_nCH4POS],YELLOW);
	}


	/*** draw offset Arrow */
	m_dcCH4.MoveTo(m_nGridX[x/2+m_nCH4_Offset],8);
	m_dcCH4.LineTo(m_nGridX[x/2+m_nCH4_Offset]-5,2);
	m_dcCH4.LineTo(m_nGridX[x/2+m_nCH4_Offset]+5,2);
	m_dcCH4.LineTo(m_nGridX[x/2+m_nCH4_Offset],8);

	m_dcCH4.SelectObject(pOldPen);
	Invalidate(TRUE); // Redraw Display
}



void CScopeView::OnClose() 
{

	
	CView::OnClose();
}

void CScopeView::Trigger()
{
		int a = pDoc->m_nTriggerOffsetLevel;
	
		/* CH1 Tigger */
		for(int i = 1;i <= 1000;i++) 
		{
			if (pDoc->m_nTriggerType == 0)
			{
				if(pDoc->m_nTriggerFormat == 1) a = pApp->m_nCH1Volt[i-1];	// Edge Triggered

				/* trigger POS to NEG */
				if (pApp->m_nCH1Volt[i] >= a && pApp->m_nCH1Volt[i+1] < a)
				{
					m_nTriggerCH1 = i;
					
					/* Disable trigger for scroll mode */
					if (pApp->m_nSampleMode == 0) m_nTriggerCH1 = 0; // disbale trigger

					i = 1001;
					break;
				}

			}
			else
			{
				if(pDoc->m_nTriggerFormat == 1) a = pApp->m_nCH1Volt[i-1];	// Edge Triggered

				/* trigger NEG to POS */
				if (pApp->m_nCH1Volt[i] <= a && pApp->m_nCH1Volt[i+1] > a)
				{
					m_nTriggerCH1 = i;
					
					/* Disable trigger for scroll mode */
					if (pApp->m_nSampleMode == 0) m_nTriggerCH1 = 0; // disbale trigger

					i = 1001;
					break;
				}
			}

		}

		/** Tigger Ch2 **/
		for(i = 1;i <= 1000;i++) 
		{
			if (pDoc->m_nTriggerType == 0)
			{
				if(pDoc->m_nTriggerFormat == 1) a = pApp->m_nCH2Volt[i-1];	// Edge Triggered

				/* trigger POS to NEG */
				if (pApp->m_nCH2Volt[i] >= a && pApp->m_nCH2Volt[i+1] < a)
				{
					m_nTriggerCH2 = i;
					
					/* Disable trigger for scroll mode */
					if (pApp->m_nSampleMode == 0) m_nTriggerCH2 = 0; // disbale trigger

					i = 1001;
					break;
				}

			}
			else
			{
				if(pDoc->m_nTriggerFormat == 1) a = pApp->m_nCH2Volt[i-1];	// Edge Triggered

				/* trigger NEG to POS */
				if (pApp->m_nCH2Volt[i] <= a && pApp->m_nCH2Volt[i+1] > a)
				{
					m_nTriggerCH2 = i;
					
					/* Disable trigger for scroll mode */
					if (pApp->m_nSampleMode == 0) m_nTriggerCH2 = 0; // disbale trigger

					i = 1001;
					break;
				}
			}

		}
	


		/** Tigger Ch3 **/
		for(i = 1;i <= 1000;i++) 
		{

			if(pDoc->m_nTriggerFormat == 1) a = pApp->m_nCH3Volt[i-1];	// Edge Triggered

			if (pDoc->m_nTriggerType == 0)
			{
				/* trigger POS to NEG */
				if (pApp->m_nCH3Volt[i] >= a && pApp->m_nCH3Volt[i+1] < a)
				{
					m_nTriggerCH3 = i;
					
					/* Disable trigger for scroll mode */
					if (pApp->m_nSampleMode == 0) m_nTriggerCH3 = 0; // disbale trigger

					i = 1001;
					break;
				}
			}
			else
			{
				if(pDoc->m_nTriggerFormat == 1) a = pApp->m_nCH3Volt[i-1];	// Edge Triggered

				/* trigger NEG to POS */
				if (pApp->m_nCH3Volt[i] <= a && pApp->m_nCH3Volt[i+1] > a)
				{
					m_nTriggerCH3 = i;
					
					/* Disable trigger for scroll mode */
					if (pApp->m_nSampleMode == 0) m_nTriggerCH3 = 0; // disbale trigger

					i = 1001;
					break;
				}
			}

		}

		/** Tigger Ch4 **/
		for(i = 1;i <= 1000;i++) 
		{
			if (pDoc->m_nTriggerType == 0)
			{

				if(pDoc->m_nTriggerFormat == 1) a = pApp->m_nCH3Volt[i-1];	// Edge Triggered

				/* trigger POS to NEG */
				if (pApp->m_nCH4Volt[i] >= a && pApp->m_nCH4Volt[i+1] < a)
				{
					if(pDoc->m_nTriggerFormat == 1) a = pApp->m_nCH4Volt[i-1];
	
					m_nTriggerCH4 = i;
					
					/* Disable trigger for scroll mode */
					if (pApp->m_nSampleMode == 0) m_nTriggerCH4 = 0; // disbale trigger

					i = 1001;
					break;
				}
			}
			else
			{
				if(pDoc->m_nTriggerFormat == 1) a = pApp->m_nCH4Volt[i-1];	// Edge Triggered

				/* trigger NEG to POS */
				if (pApp->m_nCH4Volt[i] <= a && pApp->m_nCH4Volt[i+1] > a)
				{
					m_nTriggerCH4 = i;
					
					/* Disable trigger for scroll mode */
					if (pApp->m_nSampleMode == 0) m_nTriggerCH4 = 0; // disbale trigger

					i = 1001;
					break;
				}
			}

		}

		switch (pDoc->m_nTrigger)
		{
			case 0: // trig on CH1
					m_nTriggerCH2 = m_nTriggerCH1; 
					m_nTriggerCH3 = m_nTriggerCH1; 
					m_nTriggerCH4 = m_nTriggerCH1; 
					break; 
			case 1: // trig on CH2
					m_nTriggerCH1 = m_nTriggerCH2;
					m_nTriggerCH3 = m_nTriggerCH2; 
					m_nTriggerCH4 = m_nTriggerCH2; 
					break; 
			case 2: // trig on CH3
					m_nTriggerCH1 = m_nTriggerCH3;
					m_nTriggerCH2 = m_nTriggerCH3; 
					m_nTriggerCH4 = m_nTriggerCH3; 
					break; 
			case 3: // trig on CH4
					m_nTriggerCH1 = m_nTriggerCH4;
					m_nTriggerCH2 = m_nTriggerCH4; 
					m_nTriggerCH3 = m_nTriggerCH4; 
					break; 
		}
}
